"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cli = void 0;
const dist_1 = require("@flarenetwork/flarejs/dist");
const utils_1 = require("./utils");
const constants_1 = require("./constants");
const evmAtomicTx_1 = require("./evmAtomicTx");
const pvmAtomicTx_1 = require("./pvmAtomicTx");
const addValidator_1 = require("./addValidator");
const addDelegator_1 = require("./addDelegator");
const key_1 = require("./ledger/key");
const sign_1 = require("./ledger/sign");
const forDefi_1 = require("./forDefi");
const withdrawal_1 = require("./withdrawal");
const output_1 = require("./output");
function cli(program) {
    return __awaiter(this, void 0, void 0, function* () {
        // global configurations
        program
            .option("--network <network>", "Network name (flare or costwo)", 'flare')
            .option("--ctx-file <file>", "Context file as returned by ledger commnunication tool", 'ctx.json')
            .option("--get-hashes", "Get hashes of transaction to sign")
            .option("--use-signatures", "Use hash signatures to finalize the transaction")
            .option("--use-ledger", "Use ledger to sign transactions");
        // information about the network
        program
            .command("info").description("Relevant information")
            .argument("<type>", "Type of information")
            .action((type) => __awaiter(this, void 0, void 0, function* () {
            (0, output_1.logInfo)("Getting information about the network");
            const options = program.opts();
            const ctx = (options.ctxFile) ?
                (0, constants_1.contextFile)(options.ctxFile) :
                (0, constants_1.contextEnv)(options.envPath, options.network);
            if (type == 'addresses') {
                getAddressInfo(ctx);
            }
            else if (type == 'balance') {
                yield getBalanceInfo(ctx);
            }
            else if (type == 'network') {
                getNetworkInfo(ctx);
            }
            else if (type == 'livenetwork') {
                // implement this nicely
            }
            else if (type == 'validators') {
                yield getValidatorInfo(ctx);
            }
        }));
        // moving funds from one chain to another
        program
            .command("crosschain").description("Move funds from one chain to another")
            .argument("<type>", "Type of a crosschain transaction")
            .option("-a, --amount <amount>", "Amount to transfer")
            .option("-f, --fee <fee>", "Fee of a transaction")
            .option("-id, --transaction-id <transaction-id>", "Id of the transaction to finalize")
            .action((type, options) => __awaiter(this, void 0, void 0, function* () {
            options = Object.assign(Object.assign({}, options), program.opts());
            const ctx = (options.ctxFile) ?
                (0, constants_1.contextFile)(options.ctxFile) :
                (0, constants_1.contextEnv)(options.envPath, options.network);
            if (type == 'exportCP') {
                if (options.getHashes) {
                    yield exportCP_getHashes(ctx, options.transactionId, options.amount, options.fee);
                }
                else if (options.useLedger) {
                    yield exportCP_useLedger(options.network, options.amount, options.fee);
                }
                else if (options.useSignatures) {
                    yield exportCP_useSignatures(ctx, options.transactionId);
                }
                else {
                    yield exportCP(ctx, options.amount, options.fee);
                }
            }
            else if (type == 'importCP') {
                if (options.getHashes) {
                    yield importCP_getHashes(ctx, options.transactionId);
                }
                else if (options.useSignatures) {
                    yield importCP_useSignatures(ctx, options.transactionId);
                }
                else {
                    yield importCP(ctx);
                }
            }
            else if (type == 'exportPC') {
                if (options.getHashes) {
                    //await exportPC_getHashes(ctx, options.transactionId, options.amount)
                }
                else if (options.useSignatures) {
                    //await exportPC_useSignatures(ctx, options.transactionId)
                }
                else {
                    yield exportPC(ctx, options.amount);
                }
            }
            else if (type == 'importPC') {
                if (options.getHashes) {
                    yield importPC_getHashes(ctx, options.transactionId, options.fee);
                }
                else if (options.useSignatures) {
                    yield importPC_useSignatures(ctx, options.transactionId);
                }
                else {
                    yield importPC(ctx, options.fee);
                }
            }
        }));
        // staking
        program
            .command("stake").description("Stake funds on the P-chain")
            .option("-n, --node-id <nodeID>", "The staking node's id")
            .option("-a, --amount <amount>", "Amount to stake")
            .option("-s, --start-time <start-time>", "Start time of the staking process")
            .option("-e, --end-time <end-time>", "End time of the staking process")
            .option("-id, --transaction-id <transaction-id>", "Id of the transaction to finalize")
            .action((options) => __awaiter(this, void 0, void 0, function* () {
            options = Object.assign(Object.assign({}, options), program.opts());
            const ctx = (options.ctxFile) ?
                (0, constants_1.contextFile)(options.ctxFile) :
                (0, constants_1.contextEnv)(options.envPath, options.network);
            if (options.getHashes) {
                yield stake_getHashes(ctx, options.transactionId, options.nodeId, options.amount, options.startTime, options.endTime);
            }
            else if (options.useSignatures) {
                yield stake_useSignatures(ctx, options.transactionId);
            }
            else {
                yield stake(ctx, options.nodeId, options.amount, options.startTime, options.endTime);
            }
        }));
        // delegating
        program
            .command("delegate").description("Delegate funds on the P-chain")
            .option("-n, --node-id <nodeID>", "The staking node's id")
            .option("-a, --amount <amount>", "Amount to delegate")
            .option("-s, --start-time <start-time>", "Start time of the delegation process")
            .option("-e, --end-time <end-time>", "End time of the delegation process")
            .option("-id, --transaction-id <transaction-id>", "Id of the transaction to finalize")
            .action((options) => __awaiter(this, void 0, void 0, function* () {
            options = Object.assign(Object.assign({}, options), program.opts());
            const ctx = (options.ctxFile) ?
                (0, constants_1.contextFile)(options.ctxFile) :
                (0, constants_1.contextEnv)(options.envPath, options.network);
            if (options.getHashes) {
                yield delegate_getHashes(ctx, options.transactionId, options.nodeId, options.amount, options.startTime, options.endTime);
            }
            else if (options.useSignatures) {
                yield delegate_useSignatures(ctx, options.transactionId);
            }
            else {
                yield delegate(ctx, options.nodeId, options.amount, options.startTime, options.endTime);
            }
        }));
        // forDefi signing
        program
            .command("forDefi").description("Sign with ForDefi")
            .argument("<type>", "Type of a forDefi transaction")
            .option("-id, --transaction-id <transaction-id>", "Id of the transaction to finalize")
            .option("--withdrawal", "Withdrawing funds from c-chain")
            .action((type, options) => __awaiter(this, void 0, void 0, function* () {
            options = Object.assign(Object.assign({}, options), program.opts());
            const ctx = (options.ctxFile) ? (0, constants_1.contextFile)(options.ctxFile) : (0, constants_1.contextEnv)(options.envPath, options.network);
            if (type == 'sign') {
                if (options.withdrawal) {
                    yield signForDefi(options.transactionId, options.ctxFile, true);
                }
                else {
                    yield signForDefi(options.transactionId, options.ctxFile);
                }
            }
            else if (type == 'fetch') {
                if (options.withdrawal) {
                    yield fetchForDefiTx(options.transactionId, true);
                }
                else {
                    yield fetchForDefiTx(options.transactionId);
                }
            }
        }));
        // withdrawal from c-chain
        program
            .command("withdrawal").description("Withdraw funds from c-chain")
            .option("-id, --transaction-id <transaction-id>", "Id of the transaction to finalize")
            .option("-a, --amount <amount>", "Amount to transfer")
            .option("-to, --to <to>", "Address to send funds to")
            .action((options) => __awaiter(this, void 0, void 0, function* () {
            options = Object.assign(Object.assign({}, options), program.opts());
            let ctx;
            if (options.ctxFile) {
                ctx = (0, constants_1.contextFile)(options.ctxFile);
            }
            else {
                ctx = (0, constants_1.contextEnv)(options.envPath, options.network);
            }
            if (options.getHashes) {
                yield withdraw_getHash(ctx, options.to, options.amount, options.transactionId);
            }
            else if (options.useSignatures) {
                yield withdraw_useSignature(ctx, options.transactionId);
            }
        }));
        // ledger signing
        program
            .command("init-ctx").description("Initialize context file from ledger")
            .action((options) => __awaiter(this, void 0, void 0, function* () {
            options = Object.assign(Object.assign({}, options), program.opts());
            yield (0, key_1.initContext)(key_1.DERIVATION_PATH, options.network);
            (0, output_1.logSuccess)("Context file created");
        }));
        program
            .command("sign-hash").description("Sign a transaction hash (blind signing)")
            .option("-id, --transaction-id <transaction-id>", "Id of the transaction to finalize")
            .action((options) => __awaiter(this, void 0, void 0, function* () {
            yield (0, sign_1.signId)(options.transactionId, key_1.DERIVATION_PATH, true);
            (0, output_1.logSuccess)("Transaction signed");
        }));
        program
            .command("sign").description("Sign a transaction (non-blind signing)")
            .option("-id, --transaction-id <transaction-id>", "Id of the transaction to finalize")
            .action((options) => __awaiter(this, void 0, void 0, function* () {
            yield (0, sign_1.signId)(options.transactionId, key_1.DERIVATION_PATH, false);
            (0, output_1.logSuccess)("Transaction signed");
        }));
    });
}
exports.cli = cli;
function getAddressInfo(ctx) {
    const [pubX, pubY] = ctx.publicKey;
    const compressedPubKey = (0, utils_1.compressPublicKey)(pubX, pubY).toString('hex');
    (0, output_1.log)(`P-chain address: ${ctx.pAddressBech32}`);
    (0, output_1.log)(`C-chain address hex: ${ctx.cAddressHex}`);
    (0, output_1.log)(`secp256k1 public key: 0x${compressedPubKey}`);
}
function getBalanceInfo(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        let cbalance = (new dist_1.BN(yield ctx.web3.eth.getBalance(ctx.cAddressHex))).toString();
        let pbalance = (new dist_1.BN((yield ctx.pchain.getBalance(ctx.pAddressBech32)).balance)).toString();
        cbalance = (0, utils_1.integerToDecimal)(cbalance, 18);
        pbalance = (0, utils_1.integerToDecimal)(pbalance, 9);
        (0, output_1.log)(`C-chain ${ctx.cAddressHex}: ${cbalance}`);
        (0, output_1.log)(`P-chain ${ctx.pAddressBech32}: ${pbalance}`);
    });
}
function getNetworkInfo(ctx) {
    const pchainId = ctx.pchain.getBlockchainID();
    const cchainId = ctx.cchain.getBlockchainID();
    (0, output_1.log)(`blockchainId for P-chain: ${pchainId}`);
    (0, output_1.log)(`blockchainId for C-chain: ${cchainId}`);
    (0, output_1.log)(`assetId: ${ctx.avaxAssetID}`);
}
function getValidatorInfo(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const pending = yield ctx.pchain.getPendingValidators();
        const current = yield ctx.pchain.getCurrentValidators();
        const fpending = JSON.stringify(pending, null, 2);
        const fcurrent = JSON.stringify(current, null, 2);
        (0, output_1.log)(`pending: ${fpending}`);
        (0, output_1.log)(`current: ${fcurrent}`);
    });
}
function exportCP(ctx, amount, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const famount = new dist_1.BN((0, utils_1.shiftDecimals)(amount, 9));
        const ffee = (fee === undefined) ? fee : new dist_1.BN((0, utils_1.shiftDecimals)(fee, 9));
        const { txid, usedFee } = yield (0, evmAtomicTx_1.exportTxCP)(ctx, famount, ffee);
        if (fee !== usedFee)
            (0, output_1.log)(`Used fee of ${usedFee}`);
        (0, output_1.logSuccess)(`Success! TXID: ${txid}`);
    });
}
function exportCP_getHashes(ctx, id, amount, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const famount = new dist_1.BN((0, utils_1.shiftDecimals)(amount, 9));
        const ffee = (fee === undefined) ? fee : new dist_1.BN((0, utils_1.shiftDecimals)(fee, 9));
        const unsignedTxJson = yield (0, evmAtomicTx_1.getUnsignedExportTxCP)(ctx, famount, ffee);
        (0, utils_1.saveUnsignedTxJson)(unsignedTxJson, id);
        (0, output_1.logSuccess)(`Transaction with id ${id} constructed`);
    });
}
function exportCP_useSignatures(ctx, txid) {
    return __awaiter(this, void 0, void 0, function* () {
        const { chainTxId } = yield (0, evmAtomicTx_1.issueSignedEvmTx)(ctx, (0, utils_1.readSignedTxJson)(txid));
        (0, output_1.logSuccess)(`TXID: ${chainTxId}`);
    });
}
function exportCP_useLedger(hrp, amount, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const famount = new dist_1.BN((0, utils_1.shiftDecimals)(amount, 9));
        const ffee = (fee === undefined) ? fee : new dist_1.BN((0, utils_1.shiftDecimals)(fee, 9));
        const account = yield (0, key_1.ledgerGetAccount)(key_1.DERIVATION_PATH, hrp);
        const context = (0, constants_1.getContext)(hrp, account.publicKey);
        const unsignedTxJson = yield (0, evmAtomicTx_1.getUnsignedExportTxCP)(context, famount, ffee);
        const { signature } = yield (0, sign_1.ledgerSign)(unsignedTxJson.unsignedTransactionBuffer, key_1.DERIVATION_PATH, false);
        const signedTxJson = Object.assign(Object.assign({}, unsignedTxJson), { signature });
        const { chainTxId } = yield (0, evmAtomicTx_1.issueSignedEvmTx)(context, signedTxJson);
        (0, output_1.logSuccess)(`TXID: ${chainTxId}`);
    });
}
function importCP(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const { txid } = yield (0, pvmAtomicTx_1.importTxCP)(ctx);
        (0, output_1.logSuccess)(`Transaction with id ${txid} sent to the node`);
    });
}
function importCP_getHashes(ctx, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const unsignedTxJson = yield (0, pvmAtomicTx_1.getUnsignedImportTxCP)(ctx);
        (0, utils_1.saveUnsignedTxJson)(unsignedTxJson, id);
        (0, output_1.logSuccess)(`Transaction with id ${id} constructed`);
    });
}
function importCP_useSignatures(ctx, txid) {
    return __awaiter(this, void 0, void 0, function* () {
        const { chainTxId } = yield (0, pvmAtomicTx_1.issueSignedPvmTx)(ctx, (0, utils_1.readSignedTxJson)(txid));
        (0, output_1.logSuccess)(`TXID: ${chainTxId}`);
    });
}
function exportPC(ctx, amount) {
    return __awaiter(this, void 0, void 0, function* () {
        const famount = (amount === undefined) ? amount : new dist_1.BN((0, utils_1.shiftDecimals)(amount, 9));
        const { txid } = yield (0, pvmAtomicTx_1.exportTxPC)(ctx, famount);
        (0, output_1.logSuccess)(`Transaction with id ${txid} sent to the node`);
    });
}
function importPC_getHashes(ctx, id, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const ffee = (fee === undefined) ? fee : new dist_1.BN((0, utils_1.shiftDecimals)(fee, 9));
        const unsignedTxJson = yield (0, evmAtomicTx_1.getUnsignedImportTxPC)(ctx, ffee);
        (0, utils_1.saveUnsignedTxJson)(unsignedTxJson, id);
        (0, output_1.logSuccess)(`Transaction with id ${id} constructed`);
    });
}
function importPC_useSignatures(ctx, txid) {
    return __awaiter(this, void 0, void 0, function* () {
        const { chainTxId } = yield (0, evmAtomicTx_1.issueSignedEvmTx)(ctx, (0, utils_1.readSignedTxJson)(txid));
        (0, output_1.logSuccess)(`Transaction with id ${chainTxId} sent to the node`);
    });
}
function importPC(ctx, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const ffee = (fee === undefined) ? fee : new dist_1.BN((0, utils_1.shiftDecimals)(fee, 9));
        const { txid, usedFee } = yield (0, evmAtomicTx_1.importTxPC)(ctx, ffee);
        if (fee !== usedFee)
            (0, output_1.log)(`Used fee of ${usedFee}`);
        (0, output_1.logSuccess)(`Transaction with id ${txid} sent to the node`);
    });
}
function stake(ctx, nodeID, amount, start, end) {
    return __awaiter(this, void 0, void 0, function* () {
        const famount = new dist_1.BN((0, utils_1.shiftDecimals)(amount, 9));
        const { txid } = yield (0, addValidator_1.addValidator)(ctx, nodeID, famount, new dist_1.BN(start), new dist_1.BN(end));
        (0, output_1.logSuccess)(`Transaction with id ${txid} sent to the node`);
    });
}
function stake_getHashes(ctx, id, nodeID, amount, start, end) {
    return __awaiter(this, void 0, void 0, function* () {
        const famount = new dist_1.BN((0, utils_1.shiftDecimals)(amount, 9));
        const unsignedTxJson = yield (0, addValidator_1.getUnsignedAddValidator)(ctx, nodeID, famount, new dist_1.BN(start), new dist_1.BN(end));
        (0, utils_1.saveUnsignedTxJson)(unsignedTxJson, id);
        (0, output_1.logSuccess)(`Transaction with id ${id} constructed`);
    });
}
function stake_useSignatures(ctx, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const { chainTxId } = yield (0, pvmAtomicTx_1.issueSignedPvmTx)(ctx, (0, utils_1.readSignedTxJson)(id));
        (0, output_1.logSuccess)(`TXID: ${chainTxId}`);
    });
}
function delegate(ctx, nodeID, amount, start, end) {
    return __awaiter(this, void 0, void 0, function* () {
        const famount = new dist_1.BN((0, utils_1.shiftDecimals)(amount, 9));
        const { txid } = yield (0, addDelegator_1.addDelegator)(ctx, nodeID, famount, new dist_1.BN(start), new dist_1.BN(end));
        (0, output_1.logSuccess)(`Transaction with id ${txid} sent to the node`);
    });
}
function delegate_getHashes(ctx, id, nodeID, amount, start, end) {
    return __awaiter(this, void 0, void 0, function* () {
        const famount = new dist_1.BN((0, utils_1.shiftDecimals)(amount, 9));
        const unsignedTxJson = yield (0, addDelegator_1.getUnsignedAddDelegator)(ctx, nodeID, famount, new dist_1.BN(start), new dist_1.BN(end));
        (0, utils_1.saveUnsignedTxJson)(unsignedTxJson, id);
        (0, output_1.logSuccess)(`Transaction with id ${id} constructed`);
    });
}
function delegate_useSignatures(ctx, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const signedTxJson = (0, utils_1.readSignedTxJson)(id);
        const { chainTxId } = yield (0, pvmAtomicTx_1.issueSignedPvmTx)(ctx, signedTxJson);
        (0, output_1.logSuccess)(`TXID: ${chainTxId}`);
    });
}
function signForDefi(transaction, ctx, withdrawal = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const txid = yield (0, forDefi_1.sendToForDefi)(transaction, ctx, withdrawal);
        (0, output_1.logSuccess)(`Transaction with id ${txid} sent to the node`);
    });
}
function fetchForDefiTx(transaction, withdrawal = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const signature = yield (0, forDefi_1.getSignature)(transaction, withdrawal);
        (0, output_1.logSuccess)(`Success! Signature: ${signature}`);
    });
}
function withdraw_getHash(ctx, to, amount, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const fileId = yield (0, withdrawal_1.createWithdrawalTransaction)(ctx, to, amount, id);
        (0, output_1.logSuccess)(`Transaction with id ${fileId} constructed`);
    });
}
function withdraw_useSignature(ctx, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const txId = yield (0, withdrawal_1.sendSignedWithdrawalTransaction)(ctx, id);
        (0, output_1.logSuccess)(`Transaction with id ${txId} sent to the node`);
    });
}
