"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.issueSignedEvmTx = exports.getUnsignedExportTxCP = exports.importTxPC = exports.getUnsignedImportTxPC = exports.exportTxCP = void 0;
const dist_1 = require("@flarenetwork/flarejs/dist");
const utils_1 = require("@flarenetwork/flarejs/dist/utils");
const utils_2 = require("./utils");
/**
 * Exports funds from C-chain to P-chain
 * @param ctx - context with constants initialized from user keys
 * @param amount - amount to export from C-chain to P-chain
 * @param fee - export transaction fee
 */
function exportTxCP(ctx, amount, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const txcount = yield ctx.web3.eth.getTransactionCount(ctx.cAddressHex);
        const nonce = txcount;
        const locktime = new dist_1.BN(0);
        const importFee = ctx.pchain.getDefaultTxFee();
        let unsignedTx = yield ctx.cchain.buildExportTx(amount.add(importFee), ctx.avaxAssetID, ctx.pChainBlockchainID, ctx.cAddressHex, ctx.cAddressBech32, [ctx.pAddressBech32], nonce, locktime, threshold, fee);
        if (fee === undefined) {
            const baseFeeResponse = yield ctx.cchain.getBaseFee();
            const baseFee = new dist_1.BN(parseInt(baseFeeResponse, 16) / 1e9);
            const exportCost = (0, utils_1.costExportTx)(unsignedTx);
            fee = baseFee.mul(new dist_1.BN(exportCost));
            unsignedTx = yield ctx.cchain.buildExportTx(amount.add(importFee), ctx.avaxAssetID, ctx.pChainBlockchainID, ctx.cAddressHex, ctx.cAddressBech32, [ctx.pAddressBech32], nonce, locktime, threshold, fee);
        }
        const tx = unsignedTx.sign(ctx.cKeychain);
        const txid = yield ctx.cchain.issueTx(tx);
        const usedFee = (0, utils_2.integerToDecimal)(fee.toString(), 9);
        return { txid: txid, usedFee: usedFee };
    });
}
exports.exportTxCP = exportTxCP;
/**
 * Generate unsigned import transaction from P-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param fee - import transaction fee
 */
function getUnsignedImportTxPC(ctx, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const baseFeeResponse = yield ctx.cchain.getBaseFee();
        const baseFee = new dist_1.BN(parseInt(baseFeeResponse, 16) / 1e9);
        const evmUTXOResponse = yield ctx.cchain.getUTXOs([ctx.cAddressBech32], ctx.pChainBlockchainID);
        const utxoSet = evmUTXOResponse.utxos;
        let unsignedTx = yield ctx.cchain.buildImportTx(utxoSet, ctx.cAddressHex, [ctx.cAddressBech32], ctx.pChainBlockchainID, [ctx.cAddressBech32], baseFee);
        if (fee === undefined) {
            const importCost = (0, utils_1.costImportTx)(unsignedTx);
            fee = baseFee.mul(new dist_1.BN(importCost));
            unsignedTx = yield ctx.cchain.buildImportTx(utxoSet, ctx.cAddressHex, [ctx.cAddressBech32], ctx.pChainBlockchainID, [ctx.cAddressBech32], fee);
        }
        return {
            serialization: (0, utils_2.serializeUnsignedTx)(unsignedTx),
            signatureRequests: unsignedTx.prepareUnsignedHashes(ctx.cKeychain),
            unsignedTransactionBuffer: unsignedTx.toBuffer().toString('hex'),
            usedFee: fee.toString(10)
        };
    });
}
exports.getUnsignedImportTxPC = getUnsignedImportTxPC;
/**
 * Import funds exported from P-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param fee - import transaction fee
 */
function importTxPC(ctx, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const baseFeeResponse = yield ctx.cchain.getBaseFee();
        const baseFee = new dist_1.BN(parseInt(baseFeeResponse, 16) / 1e9);
        const evmUTXOResponse = yield ctx.cchain.getUTXOs([ctx.cAddressBech32], ctx.pChainBlockchainID);
        const utxoSet = evmUTXOResponse.utxos;
        let unsignedTx = yield ctx.cchain.buildImportTx(utxoSet, ctx.cAddressHex, [ctx.cAddressBech32], ctx.pChainBlockchainID, [ctx.cAddressBech32], baseFee);
        if (fee === undefined) {
            const importCost = (0, utils_1.costImportTx)(unsignedTx);
            fee = baseFee.mul(new dist_1.BN(importCost));
            unsignedTx = yield ctx.cchain.buildImportTx(utxoSet, ctx.cAddressHex, [ctx.cAddressBech32], ctx.pChainBlockchainID, [ctx.cAddressBech32], fee);
        }
        const tx = unsignedTx.sign(ctx.cKeychain);
        const txid = yield ctx.cchain.issueTx(tx);
        const usedFee = (0, utils_2.integerToDecimal)(fee.toString(), 9);
        return { txid: txid, usedFee: usedFee };
    });
}
exports.importTxPC = importTxPC;
/**
 * Get hashes that need to get signed in order for funds to be
 * exported from P-chain to C-chain to C-chain.
 * @param ctx - context with constants initialized from user keys
 * @param amount - amount to export from C-chain to P-chain
 * @param fee - export transaction fee
 */
function getUnsignedExportTxCP(ctx, amount, fee) {
    return __awaiter(this, void 0, void 0, function* () {
        const threshold = 1;
        const txcount = yield ctx.web3.eth.getTransactionCount(ctx.cAddressHex);
        const nonce = txcount;
        const locktime = new dist_1.BN(0);
        const importFee = ctx.pchain.getDefaultTxFee();
        const args = [
            amount.add(importFee),
            ctx.avaxAssetID,
            ctx.pChainBlockchainID,
            ctx.cAddressHex,
            ctx.cAddressBech32,
            [ctx.pAddressBech32],
            nonce,
            locktime,
            threshold,
            fee
        ];
        let unsignedTx = yield ctx.cchain.buildExportTx(...args);
        if (fee === undefined) {
            const baseFeeResponse = yield ctx.cchain.getBaseFee();
            const baseFee = new dist_1.BN(parseInt(baseFeeResponse, 16) / 1e9);
            const exportCost = (0, utils_1.costExportTx)(unsignedTx);
            args[9] = baseFee.mul(new dist_1.BN(exportCost));
            unsignedTx = yield ctx.cchain.buildExportTx(...args);
        }
        return {
            serialization: (0, utils_2.serializeExportCP_args)(args),
            signatureRequests: unsignedTx.prepareUnsignedHashes(ctx.cKeychain),
            unsignedTransactionBuffer: unsignedTx.toBuffer().toString('hex'),
            usedFee: args[9].toString(10)
        };
    });
}
exports.getUnsignedExportTxCP = getUnsignedExportTxCP;
/**
 * Export funds from C-chain to P-chain by providing signed hashes
 * @param ctx - context with constants initialized from user keys
 * @param id - id associated with the transation
 */
function issueSignedEvmTx(ctx, signedTxJson) {
    return __awaiter(this, void 0, void 0, function* () {
        const signatures = Array(signedTxJson.signatureRequests.length).fill(signedTxJson.signature);
        const ecdsaSignatures = signatures.map((signature) => (0, utils_2.expandSignature)(signature));
        const unsignedTx = yield ctx.cchain.buildExportTx(...(0, utils_2.deserializeExportCP_args)(signedTxJson.serialization));
        const tx = unsignedTx.signWithRawSignatures(ecdsaSignatures, ctx.cKeychain);
        const chainTxId = yield ctx.cchain.issueTx(tx);
        return { chainTxId: chainTxId };
    });
}
exports.issueSignedEvmTx = issueSignedEvmTx;
