"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initContext = exports.ledgerGetAccount = exports.Context = exports.DERIVATION_PATH = void 0;
const fs_1 = __importDefault(require("fs"));
const hw_app_avalanche_1 = __importDefault(require("@avalabs/hw-app-avalanche"));
const hw_transport_node_hid_1 = __importDefault(require("@ledgerhq/hw-transport-node-hid"));
const utils_1 = require("./utils");
exports.DERIVATION_PATH = "m/44'/60'/0'/0/0";
class Context {
    constructor(publicKey, flareAddress, ethAddress, network) {
        this.publicKey = publicKey;
        this.flareAddress = flareAddress;
        this.ethAddress = ethAddress;
        this.network = network;
    }
}
exports.Context = Context;
function ledgerGetAccount(derivationPath, hrp) {
    return __awaiter(this, void 0, void 0, function* () {
        const transport = yield hw_transport_node_hid_1.default.open(undefined);
        const avalanche = new hw_app_avalanche_1.default(transport);
        const pubkaddr = yield avalanche.getAddressAndPubKey(derivationPath, false, hrp);
        if (pubkaddr.errorMessage != 'No errors') {
            throw Error(`Can not get address from ledger: ${pubkaddr.errorMessage}, code ${pubkaddr.returnCode}`);
        }
        return {
            publicKey: pubkaddr.publicKey.toString('hex'),
            address: pubkaddr.address
        };
    });
}
exports.ledgerGetAccount = ledgerGetAccount;
function initContext(derivationPath, hrp) {
    return __awaiter(this, void 0, void 0, function* () {
        if (fs_1.default.existsSync('ctx.json')) {
            throw Error('ctx.json already exists');
        }
        const { publicKey, address } = yield ledgerGetAccount(derivationPath, hrp);
        const data = new Context(publicKey, address, (0, utils_1.publicKeyToEthereumAddressString)(publicKey), hrp);
        fs_1.default.writeFileSync('ctx.json', JSON.stringify(data, null, 2));
    });
}
exports.initContext = initContext;
